#include "misc_aix.hpp"
#include "runtime/stubRoutines.hpp"
#include <pthread.h>

void MiscUtils::init_critsect(MiscUtils::critsect_t* cs) {
  const int rc = pthread_mutex_init(cs, NULL);
  assert0(rc == 0);
}

void MiscUtils::free_critsect(MiscUtils::critsect_t* cs) {
  const int rc = pthread_mutex_destroy(cs);
  assert0(rc == 0);
}

void MiscUtils::enter_critsect(MiscUtils::critsect_t* cs) {
  const int rc = pthread_mutex_lock(cs);
  assert0(rc == 0);
}

void MiscUtils::leave_critsect(MiscUtils::critsect_t* cs) {
  const int rc = pthread_mutex_unlock(cs);
  assert0(rc == 0);
}

bool MiscUtils::is_readable_pointer(const void* p) {
  if (!CanUseSafeFetch32()) {
    return true;
  }
  int* const aligned = (int*) align_size_down((intptr_t)p, 4);
  int cafebabe = 0xcafebabe;
  int deadbeef = 0xdeadbeef;
  return (SafeFetch32(aligned, cafebabe) != cafebabe) ||
         (SafeFetch32(aligned, deadbeef) != deadbeef);
}


